@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_inet.c,v 1.27 2004/07/23 07:25:27 alor Exp $
 */
 
 #include <ec.h>
@@ -32,10 +31,16 @@ int ip_addr_null(struct ip_addr *sa);
 int ip_addr_is_zero(struct ip_addr *sa);
 
 char *ip_addr_ntoa(struct ip_addr *sa, char *dst);
+int ip_addr_pton(char *str, struct ip_addr *addr);
 char *mac_addr_ntoa(u_char *mac, char *dst);
 int mac_addr_aton(char *str, u_char *mac);
 
-int ip_addr_is_local(struct ip_addr *sa);
+int ip_addr_is_local(struct ip_addr *sa, struct ip_addr *ifaddr);
+int ip_addr_is_broadcast(struct ip_addr *sa, struct ip_addr *ifaddr);
+int ip_addr_is_ours(struct ip_addr *);
+
+int ip_addr_get_network(struct ip_addr*, struct ip_addr*, struct ip_addr*);
+int ip_addr_get_prefix(struct ip_addr* netmask);
 
 static const char *inet_ntop4(const u_char *src, char *dst, size_t size);
 static const char *inet_ntop6(const u_char *src, char *dst, size_t size);
@@ -86,6 +91,9 @@ int ip_addr_cpy(u_char *addr, struct ip_addr *sa)
  */
 int ip_addr_cmp(struct ip_addr *sa, struct ip_addr *sb)
 {
+   if (!sa || !sb)
+      return -EINVALID;
+
    /* different type are incompatible */
    if (sa->addr_type != sb->addr_type)
       return -EINVALID;
@@ -152,7 +160,7 @@ inet_ntop4(const u_char *src, char *dst, size_t size)
    char str[IP_ASCII_ADDR_LEN];
    int n;
    
-	n = sprintf(str, "%u.%u.%u.%u", src[0], src[1], src[2], src[3]);
+	n = snprintf(str, IP_ASCII_ADDR_LEN, "%u.%u.%u.%u", src[0], src[1], src[2], src[3]);
    
    str[n] = '\0';
  
@@ -254,6 +262,21 @@ inet_ntop6(const u_char *src, char *dst, size_t size)
    return dst;
 }
 
+/* Converts character string to IP address if possible */
+int ip_addr_pton(char *str, struct ip_addr *addr)
+{
+   u_int8 buf[MAX_IP_ADDR_LEN];
+   int proto;
+
+   proto = (strchr(str, ':')) ? AF_INET6 : AF_INET;
+   
+   if(inet_pton(proto, str, buf) == 1) {
+      ip_addr_init(addr, proto, buf);
+      return ESUCCESS;
+   } else {
+      return -EINVALID;
+   }
+}
 
 /*
  * convert a MAC address to a human readable form
@@ -263,7 +286,7 @@ char *mac_addr_ntoa(u_char *mac, char *dst)
    char str[ETH_ASCII_ADDR_LEN];
    int n;
    
-	n = sprintf(str, "%02X:%02X:%02X:%02X:%02X:%02X", 
+	n = snprintf(str, ETH_ASCII_ADDR_LEN, "%02X:%02X:%02X:%02X:%02X:%02X", 
          mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    
    str[n] = '\0';
@@ -298,6 +321,55 @@ int mac_addr_aton(char *str, u_char *mac)
    return i;
 }
 
+/*
+ * returns  ESUCCESS if the ip is broadcast
+ * returns -ENOTFOUND if not
+ */
+int ip_addr_is_broadcast(struct ip_addr *sa, struct ip_addr *ifaddr)
+{
+	struct ip_addr *nw;
+	struct ip_addr *nm;
+
+	u_int32* address;
+	u_int32* netmask;
+	u_int32* network;
+	u_int32 broadcast;
+
+	switch(ntohs(sa->addr_type)) {
+		case AF_INET:
+         if(!GBL_IFACE->has_ipv4)
+            return -EINVALID;
+			nm = &GBL_IFACE->netmask;
+			nw = &GBL_IFACE->network;
+		
+         /* 255.255.255.255 is definitely broadcast */
+         if(!memcmp(sa->addr, "\xff\xff\xff\xff", IP_ADDR_LEN))
+            return ESUCCESS;
+
+			address = &ip_addr_to_int32(sa->addr);
+			netmask = &ip_addr_to_int32(nm->addr);
+			network = &ip_addr_to_int32(nw->addr);
+
+			broadcast = (*network) | ~(*netmask);
+
+			if (broadcast == *address)
+				return ESUCCESS;
+		case AF_INET6:
+			if(!GBL_IFACE->has_ipv6)
+				return -EINVALID;
+
+         /* IPv6 has no such thing as a broadcast address. The closest
+          * equivalent is the multicast address ff02::1. Packets sent to that
+          * address are delivered to all link-local nodes.
+          */
+         if(!memcpy(sa->addr, IP6_ALL_NODES, IP6_ADDR_LEN))
+            return ESUCCESS;
+         
+			break;
+	}
+
+	return -ENOTFOUND;
+}
 
 /*
  * returns ESUCCESS if the ip address is local.
@@ -307,16 +379,20 @@ int mac_addr_aton(char *str, u_char *mac)
  * if the GBL_IFACE is not filled (while reading from files)
  * returns -EINVALID.
  */
-int ip_addr_is_local(struct ip_addr *sa)
+int ip_addr_is_local(struct ip_addr *sa, struct ip_addr *ifaddr)
 {
-   struct ip_addr *nm = &GBL_IFACE->netmask;
-   struct ip_addr *nw = &GBL_IFACE->network;
-   u_int32 address;
-   u_int32 netmask;
-   u_int32 network;
-   
+   struct ip_addr *nm;
+   struct ip_addr *nw;
+   struct net_list* ip6;
+   u_int32* address;
+   u_int32* netmask;
+   u_int32* network;
+   int i;
+
    switch (ntohs(sa->addr_type)) {
       case AF_INET:
+         nm = &GBL_IFACE->netmask;
+         nw = &GBL_IFACE->network;
          /* the address 0.0.0.0 is used by DHCP and it is local for us*/
          if ( !memcmp(&sa->addr, "\x00\x00\x00\x00", ntohs(sa->addr_len)) )
             return ESUCCESS;
@@ -326,23 +402,116 @@ int ip_addr_is_local(struct ip_addr *sa)
             /* return UNKNOWN */
             return -EINVALID;
    
-         address = ip_addr_to_int32(sa->addr);
-         netmask = ip_addr_to_int32(nm->addr);
-         network = ip_addr_to_int32(nw->addr);
+         address = &ip_addr_to_int32(sa->addr);
+         netmask = &ip_addr_to_int32(nm->addr);
+         network = &ip_addr_to_int32(nw->addr);
          /* check if it is local */
-         if ((address & netmask) == network)
+         if ((*address & *netmask) == *network) {
+            if(ifaddr != NULL)
+               memcpy(ifaddr, &GBL_IFACE->ip, sizeof(*ifaddr));
             return ESUCCESS;
+         }
          break;
       case AF_INET6:
-         /* XXX - implement this */
-         NOT_IMPLEMENTED();
-         return ESUCCESS;
+         if(!GBL_IFACE->has_ipv6)
+             return -EINVALID;
+         LIST_FOREACH(ip6, &GBL_IFACE->ip6_list, next) {
+            nm = &ip6->netmask;
+            nw = &ip6->network;
+            address = &ip_addr_to_int32(sa->addr);
+            netmask = &ip_addr_to_int32(nm->addr);
+            network = &ip_addr_to_int32(nw->addr);
+
+            for(i = 0; i < IP6_ADDR_LEN / sizeof(u_int32); i++) {
+               if((address[i] & netmask[i]) != network[i])
+                  break;
+               else {
+                  if(ifaddr != NULL)
+                     memcpy(ifaddr, &ip6->ip, sizeof(*ifaddr));
+                  return ESUCCESS;
+               }
+            }
+         }
+      
          break;
    };
 
    return -ENOTFOUND;
 }
 
+int ip_addr_is_ours(struct ip_addr *ip)
+{
+   struct net_list *i;
+   switch(ntohs(ip->addr_type)) {
+      case AF_INET:
+         if(!ip_addr_cmp(ip, &GBL_IFACE->ip))
+            return EFOUND;
+         else if(!ip_addr_cmp(ip, &GBL_BRIDGE->ip))
+            return EBRIDGE;
+         else
+            return -ENOTFOUND;
+         break;
+
+      case AF_INET6:
+         LIST_FOREACH(i, &GBL_IFACE->ip6_list, next) {
+            if(!ip_addr_cmp(ip, &i->ip))
+               return EFOUND;
+         }
+         return -ENOTFOUND;
+   }
+
+   return -EINVALID;
+}
+
+int ip_addr_get_network(struct ip_addr *ip, struct ip_addr *netmask, struct ip_addr *network)
+{
+   u_int32 ip4;
+   u_int32 ip6[IP6_ADDR_LEN / sizeof(u_int32)];
+
+   if(ntohs(ip->addr_type) != ntohs(netmask->addr_type))
+      return -EINVALID;
+
+   switch(ntohs(ip->addr_type)) {
+      case AF_INET:
+         ip4 = ip_addr_to_int32(ip->addr) & ip_addr_to_int32(netmask->addr);
+         ip_addr_init(network, AF_INET, (u_char*)&ip4);
+         break;
+      case AF_INET6:
+         ip6[0] = ((u_int32*)ip->addr)[0] & ((u_int32*)netmask->addr)[0];
+         ip6[1] = ((u_int32*)ip->addr)[1] & ((u_int32*)netmask->addr)[1];
+         ip6[2] = ((u_int32*)ip->addr)[2] & ((u_int32*)netmask->addr)[2];
+         ip6[3] = ((u_int32*)ip->addr)[3] & ((u_int32*)netmask->addr)[3];
+         ip_addr_init(network, AF_INET6, (u_char*)&ip6);
+         break;
+      default:
+         BUG("Invalid addr_type");
+         return -EINVALID;
+         break;
+   }
+   return ESUCCESS;
+}
+
+int ip_addr_get_prefix(struct ip_addr* netmask)
+{
+   size_t s;
+   int i;
+   int prefix = 0;
+   u_int32* mask;
+   u_int32 x;
+
+   s = ntohs(netmask->addr_len) / sizeof(u_int32);
+
+   mask = &ip_addr_to_int32(netmask->addr);
+
+   for(i = 0; i < s; i++) {
+      x = mask[i];
+      x -= (x >> 1) & 0x55555555;
+      x = (x & 0x33333333) + ((x >> 2) & 0x33333333);
+      prefix += ((x + (x >> 4) & 0xf0f0f0f) * 0x1010101) >> 24;
+   }
+
+   return prefix;
+}
 
 /* EOF */
 
