@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_packet.c,v 1.31 2004/04/07 09:51:15 lordnaga Exp $
 */
 
 #include <ec.h>
@@ -28,6 +27,7 @@
 
 /* protos... */
 
+struct packet_object* packet_allocate_object(u_char *data, size_t len);
 inline int packet_create_object(struct packet_object *po, u_char *buf, size_t len);
 inline int packet_destroy_object(struct packet_object *po);
 int packet_disp_data(struct packet_object *po, u_char *buf, size_t len);
@@ -35,6 +35,17 @@ struct packet_object * packet_dup(struct packet_object *po, u_char flag);
 
 /* --------------------------- */
 
+struct packet_object* packet_allocate_object(u_char *data, size_t len)
+{
+   struct packet_object *po;
+
+   SAFE_CALLOC(po, 1, sizeof(struct packet_object));
+   packet_create_object(po, data, len);
+   po->flags |= PO_FORGED;
+   
+   return po;
+}
+
 /*
  * associate the buffer to the packet object
  */
@@ -54,7 +65,7 @@ inline int packet_create_object(struct packet_object *po, u_char *buf, size_t le
 /*
  * allocate the buffer for disp data
  *
- * disp data is usefull when the protocol is
+ * disp data is useful when the protocol is
  * encrypted and we want to forward the packet as is
  * but display the decrypted data.
  * decoders should decrypt data from po->DATA.data to po->DATA.disp_data
@@ -63,10 +74,13 @@ inline int packet_create_object(struct packet_object *po, u_char *buf, size_t le
 int packet_disp_data(struct packet_object *po, u_char *buf, size_t len)
 {
    /* disp_data is always null terminated */
-   if (len + 1)
+   if (len + 1) {
+      if(po->DATA.disp_data)
+        SAFE_FREE(po->DATA.disp_data);
       SAFE_CALLOC(po->DATA.disp_data, len + 1, sizeof(u_char));
-   else
+   } else {
       ERROR_MSG("packet_disp_data() negative len");
+   }
 
    po->DATA.disp_len = len;
    memcpy(po->DATA.disp_data, buf, len);
@@ -78,7 +92,7 @@ int packet_disp_data(struct packet_object *po, u_char *buf, size_t len)
  * free the packet object memory
  */
 
-inline int packet_destroy_object(struct packet_object *po)
+int packet_destroy_object(struct packet_object *po)
 {
    
    /* 
@@ -111,6 +125,12 @@ inline int packet_destroy_object(struct packet_object *po)
     * free them.
     */
    SAFE_FREE(po->DATA.disp_data);
+
+   /* if it is alloced entirely by ourselves */
+   if(po->flags & PO_FORGED) {
+      SAFE_FREE(po->packet);
+      SAFE_FREE(po);
+   }
    
    return 0;
 }
