@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_scan.c,v 1.43 2004/12/21 20:27:15 alor Exp $
 */
 
 #include <ec.h>
@@ -31,6 +30,7 @@
 #include <pthread.h>
 #include <pcap.h>
 #include <libnet.h>
+#include <time.h>
 
 /* globals */
 
@@ -58,6 +58,9 @@ static EC_THREAD_FUNC(capture_scan);
 static EC_THREAD_FUNC(scan_thread);
 static void scan_decode(u_char *param, const struct pcap_pkthdr *pkthdr, const u_char *pkt);
 
+void __init hook_init(void);
+static void hosts_list_hook(struct packet_object *po);
+
 /*******************************************/
 
 /*
@@ -95,11 +98,12 @@ void build_hosts_list(void)
       return;
 
    /* it not initialized don't make the list */
-   if (GBL_LNET->lnet == NULL)
+   if (GBL_IFACE->lnet == NULL)
       return;
 
    /* no target defined... */
    if (GBL_TARGET1->all_ip && GBL_TARGET2->all_ip &&
+       GBL_TARGET1->all_ip6 && GBL_TARGET2->all_ip6 &&
        !GBL_TARGET1->scan_all && !GBL_TARGET2->scan_all)
       return;
 
@@ -113,10 +117,10 @@ void build_hosts_list(void)
 	scan_thread(NULL);
 #else
    /* check the type of UI we are running under... */
-   if (GBL_UI->type == UI_TEXT || GBL_UI->type == UI_DAEMONIZE)
+   if (GBL_UI->type == UI_TEXT || GBL_UI->type == UI_DAEMONIZE || GBL_UI->type == UI_CURSES)
       /* in text mode and demonized call the function directly */
       scan_thread(NULL);
-   else
+   else 
       /* do the scan in a separate thread */
       ec_thread_new("scan", "scanning thread", &scan_thread, NULL);
 #endif
@@ -129,14 +133,18 @@ static EC_THREAD_FUNC(scan_thread)
 {
    pthread_t pid;
    struct hosts_list *hl;
+   struct timespec ts;
    int i = 1, ret;
    int nhosts = 0;
    int threadize = 1;
 
    DEBUG_MSG("scan_thread");
 
+   ts.tv_sec = 1;
+   ts.tv_nsec = 0;
+
    /* in text mode and demonized this function should NOT be a thread */
-   if (GBL_UI->type == UI_TEXT || GBL_UI->type == UI_DAEMONIZE)
+   if (GBL_UI->type == UI_TEXT || GBL_UI->type == UI_DAEMONIZE || GBL_UI->type == UI_CURSES )
       threadize = 0;
 
 #ifdef OS_MINGW
@@ -156,19 +164,21 @@ static EC_THREAD_FUNC(scan_thread)
     * ARP packets.
     */
    hook_add(HOOK_PACKET_ARP_RP, &get_response);
+   hook_add(HOOK_PACKET_ICMP6_NADV, &get_response);
    pid = ec_thread_new("scan_cap", "decoder module while scanning", &capture_scan, NULL);
 
    /*
-    * if at least one target is ANY, scan the whole netmask
-    * else scan only the specified targets
+    * if at least one ip target is ANY, scan the whole netmask
     *
     * the pid parameter is used to kill the thread if
     * the user request to stop the scan.
+    *
+    * FIXME: ipv4 host gets scanned twice if in target list
     */
-   if (GBL_TARGET1->scan_all || GBL_TARGET2->scan_all)
+   if(GBL_TARGET1->all_ip || GBL_TARGET2->all_ip) {
       scan_netmask(pid);
-   else
-      scan_targets(pid);
+   }
+   scan_targets(pid);
 
    /*
     * free the temporary array for random computations
@@ -180,11 +190,17 @@ static EC_THREAD_FUNC(scan_thread)
     * wait for some delayed packets...
     * the other thread is listening for ARP pachets
     */
-   sleep(1);
+
+#if defined(OS_WINDOWS)
+   usleep(1000); //1 msec
+#else
+   nanosleep(&ts, NULL);
+#endif
 
    /* destroy the thread and remove the hook function */
    ec_thread_destroy(pid);
    hook_del(HOOK_PACKET_ARP, &get_response);
+   hook_del(HOOK_PACKET_ICMP6_NADV, &get_response);
 
    /* count the hosts and print the message */
    LIST_FOREACH(hl, &GBL_HOSTLIST, next) {
@@ -196,6 +212,9 @@ static EC_THREAD_FUNC(scan_thread)
 
    INSTANT_USER_MSG("%d hosts added to the hosts list...\n", nhosts);
 
+   /* update host list*/
+   ui_update(UI_UPDATE_HOSTLIST);
+
    /*
     * resolve the hostnames only if we are scanning
     * the lan. when loading from file, hostnames are
@@ -262,7 +281,7 @@ static EC_THREAD_FUNC(capture_scan)
 
    ec_thread_init();
 
-   pcap_loop(GBL_PCAP->pcap, -1, scan_decode, EC_THREAD_PARAM);
+   pcap_loop(GBL_IFACE->pcap, -1, scan_decode, EC_THREAD_PARAM);
 
    return NULL;
 }
@@ -292,8 +311,8 @@ static void scan_decode(u_char *param, const struct pcap_pkthdr *pkthdr, const u
    memcpy(&po.ts, &pkthdr->ts, sizeof(struct timeval));
 
    /*
-    * in this special parsing, the packet must be ingored by
-    * application layer, leave this un touched.
+    * in this special parsing, the packet must be ignored by
+    * application layer, leave this untouched.
     */
    po.flags |= PO_DONT_DISSECT;
 
@@ -356,6 +375,12 @@ static void scan_netmask(pthread_t pid)
    struct ip_list *e, *tmp;
    char title[100];
 
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_nsec = GBL_CONF->arp_storm_delay * 1000;
+   tm.tv_sec = 0;
+#endif
+
    netmask = ip_addr_to_int32(&GBL_IFACE->netmask.addr);
    myip = ip_addr_to_int32(&GBL_IFACE->ip.addr);
 
@@ -410,7 +435,12 @@ static void scan_netmask(pthread_t pid)
       }
 
       /* wait for a delay */
+#if defined(OS_WINDOWS)
       usleep(GBL_CONF->arp_storm_delay * 1000);
+#else
+      nanosleep(&tm, NULL);
+#endif
+
    }
 
    /* delete the temporary list */
@@ -418,6 +448,8 @@ static void scan_netmask(pthread_t pid)
       LIST_REMOVE(e, next);
       SAFE_FREE(e);
    }
+
+   DEBUG_MSG("scan_netmask: Complete");
 }
 
 
@@ -429,6 +461,14 @@ static void scan_targets(pthread_t pid)
    int nhosts = 0, found, n = 1, ret;
    struct ip_list *e, *i, *m, *tmp;
    char title[100];
+   struct ip_addr ip;
+   struct ip_addr bc;
+
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_nsec = GBL_CONF->arp_storm_delay * 1000;
+   tm.tv_sec = 0;
+#endif
 
    DEBUG_MSG("scan_targets: merging targets...");
 
@@ -449,32 +489,58 @@ static void scan_targets(pthread_t pid)
       /* add to the list randomly */
       random_list(e, nhosts);
    }
+   LIST_FOREACH(i, &GBL_TARGET1->ip6, next) {
+
+      SAFE_CALLOC(e, 1, sizeof(struct ip_list));
+      memcpy(&e->ip, &i->ip, sizeof(struct ip_addr));
+      nhosts++;
+
+      random_list(e, nhosts);
+   }
 
    /* then merge the target2 ips */
    LIST_FOREACH(i, &GBL_TARGET2->ips, next) {
 
-      SAFE_CALLOC(e, 1, sizeof(struct ip_list));
-
-      memcpy(&e->ip, &i->ip, sizeof(struct ip_addr));
-
       found = 0;
 
       /* search if it is already in the list */
       LIST_FOREACH(m, &ip_list_head, next)
          if (!ip_addr_cmp(&m->ip, &i->ip)) {
             found = 1;
-            SAFE_FREE(e);
             break;
          }
 
       /* add it */
       if (!found) {
+         SAFE_CALLOC(e, 1, sizeof(struct ip_list));
+         memcpy(&e->ip, &i->ip, sizeof(struct ip_addr));
+
          nhosts++;
          /* add to the list randomly */
          random_list(e, nhosts);
       }
    }
 
+   LIST_FOREACH(i, &GBL_TARGET2->ip6, next) {
+      found = 0;
+
+      LIST_FOREACH(m, &ip_list_head, next)
+         if (!ip_addr_cmp(&m->ip, &i->ip)) {
+            found = 1;
+            break;
+         }
+
+      if (!found) {
+         SAFE_CALLOC(e, 1, sizeof(struct ip_list));
+         memcpy(&e->ip, &i->ip, sizeof(struct ip_addr));
+
+         nhosts++;
+         /* add to the list randomly */
+         random_list(e, nhosts);
+      }
+   }
+
+
    DEBUG_MSG("scan_targets: %d hosts to be scanned", nhosts);
 
    /* don't scan if there are no hosts */
@@ -487,7 +553,18 @@ static void scan_targets(pthread_t pid)
    /* and now scan the LAN */
    LIST_FOREACH(e, &ip_list_head, next) {
       /* send the arp request */
-      send_arp(ARPOP_REQUEST, &GBL_IFACE->ip, GBL_IFACE->mac, &e->ip, MEDIA_BROADCAST);
+      switch(ntohs(e->ip.addr_type)) {
+         case AF_INET:
+            send_arp(ARPOP_REQUEST, &GBL_IFACE->ip, GBL_IFACE->mac, &e->ip, MEDIA_BROADCAST);
+            break;
+#ifdef WITH_IPV6
+         case AF_INET6:
+            ip_addr_is_local(&e->ip, &ip);
+            ip_addr_init(&bc, AF_INET6, IP6_ALL_NODES);
+            send_icmp6_nsol(&ip, &bc, &e->ip, GBL_IFACE->mac);
+            break;
+#endif
+      }
 
       /* update the progress bar */
       ret = ui_progress(title, n++, nhosts);
@@ -498,6 +575,7 @@ static void scan_targets(pthread_t pid)
          /* destroy the capture thread and remove the hook function */
          ec_thread_destroy(pid);
          hook_del(HOOK_PACKET_ARP, &get_response);
+         hook_del(HOOK_PACKET_ICMP6_NADV, &get_response);
          /* delete the temporary list */
          LIST_FOREACH_SAFE(e, &ip_list_head, next, tmp) {
             LIST_REMOVE(e, next);
@@ -508,7 +586,12 @@ static void scan_targets(pthread_t pid)
       }
 
       /* wait for a delay */
-      usleep(GBL_CONF->arp_storm_delay * 1000);
+#if defined(OS_WINDOWS)
+      usleep(GBL_CONF->arp_storm_delay * 100);
+#else
+      nanosleep(&tm, NULL);
+#endif
+
    }
 
    /* delete the temporary list */
@@ -526,8 +609,10 @@ int scan_load_hosts(char *filename)
 {
    FILE *hf;
    int nhosts;
-   char ip[16], mac[18], name[128];
-   struct in_addr tip;
+   char ip[MAX_ASCII_ADDR_LEN];
+   char mac[ETH_ASCII_ADDR_LEN];
+   char name[MAX_HOSTNAME_LEN];
+   u_int8 tip[MAX_IP_ADDR_LEN];
    struct ip_addr hip;
    u_int8 hmac[MEDIA_ADDR_LEN];
 
@@ -543,21 +628,22 @@ int scan_load_hosts(char *filename)
    /* XXX - adapt to IPv6 */
    /* read the file */
    for (nhosts = 0; !feof(hf); nhosts++) {
+      int proto;
 
-      if (fscanf(hf,"%15s %17s %127s\n", ip, mac, name) != 3 ||
+      if (fscanf(hf, "%"EC_TOSTRING(MAX_ASCII_ADDR_LEN)"s %"EC_TOSTRING(ETH_ASCII_ADDR_LEN)"s %"EC_TOSTRING(MAX_HOSTNAME_LEN)"s\n", ip, mac, name) != 3 ||
          *ip == '#' || *mac == '#' || *name == '#')
          continue;
 
-
       /* convert to network */
       mac_addr_aton(mac, hmac);
 
-      if (inet_aton(ip, &tip) == 0) {
+      proto = (strchr(ip, ':')) ? AF_INET6 : AF_INET;
+      if (!inet_pton(proto, ip, tip)) {
          del_hosts_list();
          SEMIFATAL_ERROR("Bad parsing on line %d", nhosts + 1);
       }
 
-      ip_addr_init(&hip, AF_INET, (char *)&tip);
+      ip_addr_init(&hip, proto, (char *)tip);
 
       /* wipe the null hostname */
       if (!strcmp(name, "-"))
@@ -694,6 +780,31 @@ static void random_list(struct ip_list *e, int max)
 
 }
 
+void __init hook_init(void)
+{
+   hook_add(HOOK_PACKET_IP, &hosts_list_hook);
+   hook_add(HOOK_PACKET_IP6, &hosts_list_hook);
+}
+
+/*
+ * This function adds local nodes to the global hosts list.
+ * Its quite slow and I don't have any better ideas at the moment.
+ */
+static void hosts_list_hook(struct packet_object *po)
+{
+   switch(ip_addr_is_ours(&po->L3.src)) {
+      case EFOUND:
+      case EBRIDGE:
+         return;
+   }
+
+   if(ip_addr_is_local(&po->L3.src, NULL) == ESUCCESS) {
+      add_host(&po->L3.src, po->L2.src, NULL);
+   }
+
+   return;
+}
+
 /* EOF */
 
 // vim:ts=3:expandtab
