@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_sniff_bridge.c,v 1.18 2004/11/04 10:29:06 alor Exp $
 */
 
 #include <ec.h>
@@ -64,10 +63,10 @@ void start_bridge_sniff(void)
    }
 
    /* create the thread for packet capture */
-   ec_thread_new("capture", "pcap handler and packet decoder", &capture, GBL_OPTIONS->iface);
+   capture_start(GBL_IFACE);
    
    /* create the thread for packet capture on the bridged interface */
-   ec_thread_new("bridge", "pcap handler and packet decoder", &capture_bridge, GBL_OPTIONS->iface_bridge);
+   capture_start(GBL_BRIDGE);
 
    GBL_SNIFF->active = 1;
 }
@@ -86,14 +85,9 @@ void stop_bridge_sniff(void)
       return;
    }
   
-   /* get the pid and kill it */
-   pid = ec_thread_getpid("capture");
-   if (!pthread_equal(pid, EC_PTHREAD_NULL))
-      ec_thread_destroy(pid);
-   
-   pid = ec_thread_getpid("bridge");
-   if (!pthread_equal(pid, EC_PTHREAD_NULL))
-      ec_thread_destroy(pid);
+   /* kill it */
+   capture_stop(GBL_IFACE);
+   capture_stop(GBL_BRIDGE);
 
    USER_MSG("Bridged sniffing was stopped.\n");
 
