@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_sslwrap.c,v 1.55 2004/09/14 07:58:17 alor Exp $
 */
 
 #include <ec.h>
@@ -35,7 +34,14 @@
 #ifndef OS_WINDOWS
    #include <sys/wait.h>
 #endif
+
+#ifdef OS_LINUX
+   #include <linux/netfilter_ipv4.h>
+#endif
+
 #include <fcntl.h>
+#include <time.h>
+#include <pthread.h>
 
 #ifdef HAVE_OPENSSL
 
@@ -93,7 +99,7 @@ struct accepted_entry {
 };
 
 /* Session identifier 
- * It has to be even-lenghted for session hash matching */
+ * It has to be of even length for session hash matching */
 struct sslw_ident {
    u_int32 magic;
       #define SSLW_MAGIC  0x0501e77e
@@ -104,7 +110,12 @@ struct sslw_ident {
 #define SSLW_IDENT_LEN sizeof(struct sslw_ident)
 
 #define SSLW_RETRY 5
-#define SSLW_WAIT 10000
+#define SSLW_WAIT 10 /* 10 seconds */
+
+#if defined(OS_DARWIN) || defined(OS_BSD)
+#define SSLW_SET "20"
+#endif
+
 
 #define TSLEEP (50*1000) /* 50 milliseconds */
 
@@ -145,6 +156,7 @@ static int sslw_remove_redirect(u_int16 sport, u_int16 dport);
 static void ssl_wrap_fini(void);
 static int sslw_ssl_connect(SSL *ssl_sk);
 static int sslw_ssl_accept(SSL *ssl_sk);
+static int sslw_remove_sts(struct packet_object *po);
 
 #endif /* HAVE_OPENSSL */
 
@@ -237,11 +249,19 @@ void ssl_wrap_init(void)
 #ifdef HAVE_OPENSSL
 static void ssl_wrap_fini(void)
 {
-   struct listen_entry *le;
+   struct listen_entry *le, *old;
 
+   DEBUG_MSG("ATEXIT: ssl_wrap_fini");
    /* remove every redirect rule */   
-   LIST_FOREACH(le, &listen_ports, next)
+   LIST_FOREACH_SAFE(le, &listen_ports, next, old) {
       sslw_remove_redirect(le->sslw_port, le->redir_port);
+      LIST_REMOVE(le, next);
+      SAFE_FREE(le);
+   }
+
+   SSL_CTX_free(ssl_ctx_server);
+   SSL_CTX_free(ssl_ctx_client);
+
 }
 #endif
 
@@ -264,7 +284,7 @@ EC_THREAD_FUNC(sslw_start)
    return NULL;
 #else
    
-   /* disabled if not accressive */
+   /* disabled if not aggressive */
    if (!GBL_CONF->aggressive_dissectors)
       return NULL;
    
@@ -312,8 +332,9 @@ EC_THREAD_FUNC(sslw_start)
             /* Set the peer (client) in the connection list entry */
             ae->port[SSL_CLIENT] = client_sin.sin_port;
             ip_addr_init(&(ae->ip[SSL_CLIENT]), AF_INET, (char *)&(client_sin.sin_addr.s_addr));
-	    
-            ec_thread_new("sslw_child", "ssl child", &sslw_child, ae);
+	   
+            /* create a detached thread */ 
+            ec_thread_new_detached("sslw_child", "ssl child", &sslw_child, ae, 1);
          }
    }
 
@@ -335,7 +356,7 @@ static void sslw_hook_handled(struct packet_object *po)
    /* We have nothing to do with this packet */
    if (!sslw_is_ssl(po))
       return;
-      
+     
    /* If it's an ssl packet don't forward */
    po->flags |= PO_DROPPED;
    
@@ -346,9 +367,13 @@ static void sslw_hook_handled(struct packet_object *po)
 	
       sslw_create_session(&s, PACKET);
 
+#ifndef OS_LINUX
       /* Remember the real destination IP */
       memcpy(s->data, &po->L3.dst, sizeof(struct ip_addr));
       session_put(s);
+#else
+	SAFE_FREE(s); /* Just get rid of it */
+#endif
    } else /* Pass only the SYN for conntrack */
       po->flags |= PO_IGNORE;
 }
@@ -368,14 +393,18 @@ static int sslw_insert_redirect(u_int16 sport, u_int16 dport)
    if (GBL_CONF->redir_command_on == NULL)
       return -EFATAL;
    
-   sprintf(asc_sport, "%u", sport);
-   sprintf(asc_dport, "%u", dport);
+   snprintf(asc_sport, 16, "%u", sport);
+   snprintf(asc_dport, 16, "%u", dport);
 
    /* make the substitutions in the script */
    command = strdup(GBL_CONF->redir_command_on);
    str_replace(&command, "%iface", GBL_OPTIONS->iface);
    str_replace(&command, "%port", asc_sport);
    str_replace(&command, "%rport", asc_dport);
+
+#if defined(OS_DARWIN) || defined(OS_BSD)
+   str_replace(&command, "%set", SSLW_SET);
+#endif
    
    DEBUG_MSG("sslw_insert_redirect: [%s]", command);
    
@@ -425,14 +454,18 @@ static int sslw_remove_redirect(u_int16 sport, u_int16 dport)
    if (GBL_CONF->redir_command_off == NULL)
       return -EFATAL;
    
-   sprintf(asc_sport, "%u", sport);
-   sprintf(asc_dport, "%u", dport);
+   snprintf(asc_sport, 16, "%u", sport);
+   snprintf(asc_dport, 16, "%u", dport);
 
    /* make the substitutions in the script */
    command = strdup(GBL_CONF->redir_command_off);
    str_replace(&command, "%iface", GBL_OPTIONS->iface);
    str_replace(&command, "%port", asc_sport);
    str_replace(&command, "%rport", asc_dport);
+
+#if defined(OS_DARWIN) || defined(OS_BSD)
+   str_replace(&command, "%set", SSLW_SET);
+#endif
    
    DEBUG_MSG("sslw_remove_redirect: [%s]", command);
    
@@ -545,6 +578,12 @@ static int sslw_ssl_connect(SSL *ssl_sk)
 { 
    int loops = (GBL_CONF->connect_timeout * 10e5) / TSLEEP;
    int ret, ssl_err;
+
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_sec = 0;
+   tm.tv_nsec = TSLEEP * 1000;
+#endif
    
    do {
       /* connect to the server */
@@ -558,7 +597,11 @@ static int sslw_ssl_connect(SSL *ssl_sk)
          return -EINVALID;
       
       /* sleep a quirk of time... */
+#if defined(OS_WINDOWS)
       usleep(TSLEEP);
+#else
+      nanosleep(&tm, NULL);
+#endif
    } while(loops--);
 
    return -EINVALID;
@@ -573,6 +616,12 @@ static int sslw_ssl_accept(SSL *ssl_sk)
 { 
    int loops = (GBL_CONF->connect_timeout * 10e5) / TSLEEP;
    int ret, ssl_err;
+
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_sec = 0;
+   tm.tv_nsec = TSLEEP * 1000;
+#endif
    
    do {
       /* accept the ssl connection */
@@ -586,7 +635,11 @@ static int sslw_ssl_accept(SSL *ssl_sk)
          return -EINVALID;
       
       /* sleep a quirk of time... */
+#if defined(OS_WINDOWS)
       usleep(TSLEEP);
+#else
+      nanosleep(&tm, NULL);
+#endif
    } while(loops--);
 
    return -EINVALID;
@@ -601,6 +654,7 @@ static int sslw_ssl_accept(SSL *ssl_sk)
  */   
 static int sslw_sync_ssl(struct accepted_entry *ae) 
 {   
+
    X509 *server_cert;
    
    ae->ssl[SSL_SERVER] = SSL_new(ssl_ctx_server);
@@ -618,18 +672,22 @@ static int sslw_sync_ssl(struct accepted_entry *ae)
       return -EINVALID;
    }
 
-   /* Create the fake certificate */
-   ae->cert = sslw_create_selfsigned(server_cert);  
-   X509_free(server_cert);
+   if (!GBL_OPTIONS->ssl_cert) {
+   	/* Create the fake certificate */
+   	ae->cert = sslw_create_selfsigned(server_cert);  
+   	X509_free(server_cert);
 
-   if (ae->cert == NULL)
-      return -EINVALID;
-   
-   SSL_use_certificate(ae->ssl[SSL_CLIENT], ae->cert);
+   	if (ae->cert == NULL)
+      		return -EINVALID;
+
+   	SSL_use_certificate(ae->ssl[SSL_CLIENT], ae->cert);
+
+   }
    
    if (sslw_ssl_accept(ae->ssl[SSL_CLIENT]) != ESUCCESS) 
       return -EINVALID;
 
+
    return ESUCCESS;   
 }
 
@@ -640,6 +698,11 @@ static int sslw_sync_ssl(struct accepted_entry *ae)
  */
 static int sslw_get_peer(struct accepted_entry *ae)
 {
+
+/* If on Linux, we can just get the SO_ORIGINAL_DST from getsockopt() no need for this loop
+   nonsense.
+*/
+#ifndef OS_LINUX
    struct ec_session *s = NULL;
    struct packet_object po;
    void *ident = NULL;
@@ -651,13 +714,23 @@ static int sslw_get_peer(struct accepted_entry *ae)
    po.L4.dst = ae->port[SSL_SERVER];
    
    sslw_create_ident(&ident, &po);
+
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_sec = SSLW_WAIT;
+   tm.tv_nsec = 0;
+#endif
    
    /* 
     * A little waiting loop because the sniffing thread , 
     * which creates the session, may be slower than this
     */
    for (i=0; i<SSLW_RETRY && session_get_and_del(&s, ident, SSLW_IDENT_LEN)!=ESUCCESS; i++)
+#if defined(OS_WINDOWS)
       usleep(SSLW_WAIT);
+#else
+      nanosleep(&tm, NULL); 
+#endif /* OS_WINDOWS */
 
    if (i==SSLW_RETRY) {
       SAFE_FREE(ident);
@@ -670,7 +743,14 @@ static int sslw_get_peer(struct accepted_entry *ae)
    SAFE_FREE(s->data);
    SAFE_FREE(s);
    SAFE_FREE(ident);
+#else
+   struct sockaddr_in sa_in;
+   socklen_t sa_in_sz = sizeof(struct sockaddr_in);
 
+   getsockopt(ae->fd[SSL_CLIENT], SOL_IP, SO_ORIGINAL_DST, (struct sockaddr*)&sa_in, &sa_in_sz);
+
+   ip_addr_init(&(ae->ip[SSL_SERVER]), AF_INET, (char *)&(sa_in.sin_addr.s_addr));
+#endif
    return ESUCCESS;
 }
 
@@ -694,6 +774,7 @@ static int sslw_connect_server(struct accepted_entry *ae)
    /* Standard connection to the server */
    if (!dest_ip || (ae->fd[SSL_SERVER] = open_socket(dest_ip, ntohs(ae->port[SSL_SERVER]))) < 0) {
       SAFE_FREE(dest_ip);   
+      DEBUG_MSG("Could not open socket");
       return -EINVALID;
    }
    
@@ -750,11 +831,15 @@ static int sslw_read_data(struct accepted_entry *ae, u_int32 direction, struct p
 
    /* NULL terminate the data buffer */
    po->DATA.data[po->DATA.len] = 0;
- 
+
+   /* remove STS header */ 
+   if (direction == SSL_SERVER)
+       sslw_remove_sts(po);
+
    /* create the buffer to be displayed */
    packet_destroy_object(po);
    packet_disp_data(po, po->DATA.data, po->DATA.len);
-  
+   
    return ESUCCESS;
 }
 
@@ -770,6 +855,14 @@ static int sslw_write_data(struct accepted_entry *ae, u_int32 direction, struct
 
    packet_len = (int32)(po->DATA.len + po->DATA.inject_len);
    p_data = po->DATA.data;
+
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_sec = 1;
+   tm.tv_nsec = 0;
+#else
+   int timeout = 1000;
+#endif
    
    if (packet_len == 0)
       return ESUCCESS;
@@ -810,7 +903,11 @@ static int sslw_write_data(struct accepted_entry *ae, u_int32 direction, struct
       
       /* XXX - Set a proper sleep time */
       if (not_written)
-         usleep(1000);
+#if defined(OS_WINDOWS)
+         usleep(timeout);
+#else
+         nanosleep(&tm, NULL);
+#endif
 	 	 
    } while (not_written);
          
@@ -846,7 +943,7 @@ static void sslw_parse_packet(struct accepted_entry *ae, u_int32 direction, stru
    gettimeofday(&po->ts, NULL);
 
    /* calculate if the dest is local or not */
-   switch (ip_addr_is_local(&PACKET->L3.src)) {
+   switch (ip_addr_is_local(&PACKET->L3.src, NULL)) {
       case ESUCCESS:
          PACKET->PASSIVE.flags &= ~FP_HOST_NONLOCAL;
          PACKET->PASSIVE.flags |= FP_HOST_LOCAL;
@@ -884,7 +981,8 @@ static void sslw_wipe_connection(struct accepted_entry *ae)
    if (ae->cert)
       X509_free(ae->cert);
 
-   SAFE_FREE(ae);
+   if(ae)
+     SAFE_FREE(ae);
 }
 
 /* 
@@ -898,10 +996,14 @@ static void sslw_initialize_po(struct packet_object *po, u_char *p_data)
     * XXX - Be sure to not modify these len.
     */
    memset(po, 0, sizeof(struct packet_object));
-   if (p_data == NULL)
+   if (p_data == NULL) {
       SAFE_CALLOC(po->DATA.data, 1, UINT16_MAX);
-   else
-      po->DATA.data = p_data;
+   } else {
+      if (po->DATA.data != p_data) {
+      	  SAFE_FREE(po->DATA.data);
+          po->DATA.data = p_data;
+      }
+   }
       
    po->L2.header  = po->DATA.data; 
    po->L3.header  = po->DATA.data;
@@ -923,14 +1025,14 @@ static void sslw_initialize_po(struct packet_object *po, u_char *p_data)
 static X509 *sslw_create_selfsigned(X509 *server_cert)
 {   
    X509 *out_cert;
-//   X509_EXTENSION *ext;
-//   int index = 0;
+   X509_EXTENSION *ext;
+   int index = 0;
    
    if ((out_cert = X509_new()) == NULL)
       return NULL;
       
    /* Set out public key, real server name... */
-   X509_set_version(out_cert, 0x2);
+   X509_set_version(out_cert, X509_get_version(server_cert));
    X509_set_serialNumber(out_cert, X509_get_serialNumber(server_cert));   
    X509_set_notBefore(out_cert, X509_get_notBefore(server_cert));
    X509_set_notAfter(out_cert, X509_get_notAfter(server_cert));
@@ -939,9 +1041,8 @@ static X509 *sslw_create_selfsigned(X509 *server_cert)
    X509_set_issuer_name(out_cert, X509_get_issuer_name(server_cert));  
    
    /* Modify the issuer a little bit */ 
-   X509_NAME_add_entry_by_txt(X509_get_issuer_name(out_cert), "L", MBSTRING_ASC, " ", -1, -1, 0);
+   //X509_NAME_add_entry_by_txt(X509_get_issuer_name(out_cert), "L", MBSTRING_ASC, " ", -1, -1, 0);
 
-/*
    index = X509_get_ext_by_NID(server_cert, NID_authority_key_identifier, -1);
    if (index >=0) {
       ext = X509_get_ext(server_cert, index);
@@ -951,7 +1052,6 @@ static X509 *sslw_create_selfsigned(X509 *server_cert)
          X509_add_ext(out_cert, ext, -1);
       }
    }
-*/
 
    /* Self-sign our certificate */
    if (!X509_sign(out_cert, global_pk, EVP_sha1())) {
@@ -977,12 +1077,33 @@ static void sslw_init(void)
    ssl_ctx_client = SSL_CTX_new(SSLv23_server_method());
    ssl_ctx_server = SSL_CTX_new(SSLv23_client_method());
 
-   /* Get our private key from our cert file */
-   if (SSL_CTX_use_PrivateKey_file(ssl_ctx_client, INSTALL_DATADIR "/" EC_PROGRAM "/" CERT_FILE, SSL_FILETYPE_PEM) == 0) {
-      DEBUG_MSG("sslw -- SSL_CTX_use_PrivateKey_file -- trying ./share/%s",  CERT_FILE);
+   ON_ERROR(ssl_ctx_client, NULL, "Could not create client SSL CTX");
+   ON_ERROR(ssl_ctx_server, NULL, "Could not create server SSL CTX");
 
-      if (SSL_CTX_use_PrivateKey_file(ssl_ctx_client, "./share/" CERT_FILE, SSL_FILETYPE_PEM) == 0)
-         FATAL_ERROR("Can't open \"./share/%s\" file : %s", CERT_FILE, strerror(errno));
+   if(GBL_OPTIONS->ssl_pkey) {
+	/* Get our private key from the file specified from cmd-line */
+	DEBUG_MSG("Using custom private key %s", GBL_OPTIONS->ssl_pkey);
+	if (SSL_CTX_use_PrivateKey_file(ssl_ctx_client, GBL_OPTIONS->ssl_pkey, SSL_FILETYPE_PEM) == 0) {
+		FATAL_ERROR("Can't open \"%s\" file : %s", GBL_OPTIONS->ssl_pkey, strerror(errno));
+	}
+
+	if (GBL_OPTIONS->ssl_cert) {
+		if (SSL_CTX_use_certificate_file(ssl_ctx_client, GBL_OPTIONS->ssl_cert, SSL_FILETYPE_PEM) == 0) {
+			FATAL_ERROR("Can't open \"%s\" file : %s", GBL_OPTIONS->ssl_cert, strerror(errno));
+		}
+
+		if (!SSL_CTX_check_private_key(ssl_ctx_client)) {
+			FATAL_ERROR("Certificate \"%s\" does not match private key \"%s\"", GBL_OPTIONS->ssl_cert, GBL_OPTIONS->ssl_pkey);
+		}
+	}
+   } else {
+   	/* Get our private key from our cert file */
+   	if (SSL_CTX_use_PrivateKey_file(ssl_ctx_client, INSTALL_DATADIR "/" EC_PROGRAM "/" CERT_FILE, SSL_FILETYPE_PEM) == 0) {
+      		DEBUG_MSG("sslw -- SSL_CTX_use_PrivateKey_file -- trying ./share/%s",  CERT_FILE);
+
+      		if (SSL_CTX_use_PrivateKey_file(ssl_ctx_client, "./share/" CERT_FILE, SSL_FILETYPE_PEM) == 0)
+         		FATAL_ERROR("Can't open \"./share/%s\" file : %s", CERT_FILE, strerror(errno));
+   	}
    }
 
    dummy_ssl = SSL_new(ssl_ctx_client);
@@ -996,18 +1117,32 @@ static void sslw_init(void)
 /* 
  * SSL thread child function.
  */
+
 EC_THREAD_FUNC(sslw_child)
 {
    struct packet_object po;
    int direction, ret_val, data_read;
    struct accepted_entry *ae;
+#if !defined(OS_WINDOWS)
+   struct timespec tm;
+   tm.tv_sec = 0;
+   tm.tv_nsec = 3000*1000;
+#else
+   int timeout = 3000;
+#endif
 
    ae = (struct accepted_entry *)args;
    ec_thread_init();
- 
+
+
+   /* We don't want this to accidentally close STDIN */
+   ae->fd[SSL_SERVER] = -1;
+
    /* Contact the real server */
    if (sslw_sync_conn(ae) == -EINVALID) {
-      close_socket(ae->fd[SSL_CLIENT]);
+      if (ae->fd[SSL_CLIENT] != -1)
+         close_socket(ae->fd[SSL_CLIENT]);
+      DEBUG_MSG("FAILED TO FIND PEER");
       SAFE_FREE(ae);
       ec_thread_exit();
    }	    
@@ -1023,22 +1158,28 @@ EC_THREAD_FUNC(sslw_child)
    po.len = 64;
    po.L4.flags = (TH_SYN | TH_ACK);
    packet_disp_data(&po, po.DATA.data, po.DATA.len);
+
    sslw_parse_packet(ae, SSL_SERVER, &po);
    sslw_initialize_po(&po, po.DATA.data);
    
    LOOP {
+
       data_read = 0;
       for(direction=0; direction<2; direction++) {
+
          ret_val = sslw_read_data(ae, direction, &po);
          BREAK_ON_ERROR(ret_val,ae,po);
 	 
          /* if we have data to read */
          if (ret_val == ESUCCESS) {
             data_read = 1;
+
+
             sslw_parse_packet(ae, direction, &po);
+
             if (po.flags & PO_DROPPED)
                continue;
-
+	
             ret_val = sslw_write_data(ae, !direction, &po);
             BREAK_ON_ERROR(ret_val,ae,po);
 	    
@@ -1053,14 +1194,70 @@ EC_THREAD_FUNC(sslw_child)
       }
 
       /* XXX - Set a proper sleep time */
+      /* Should we poll both fd's instead of guessing and sleeping? */
       if (!data_read)
-         usleep(3000);
+#if defined(OS_WINDOWS)
+        usleep(timeout);
+#else
+        nanosleep(&tm, NULL);
+#endif
    }
 
    return NULL;
 }
 
 
+static int sslw_remove_sts(struct packet_object *po)
+{
+	u_char *ptr;
+	u_char *end;
+	u_char *h_end;
+	size_t len = po->DATA.len;
+	size_t slen = strlen("\r\nStrict-Transport-Security:");
+
+	if (!memmem(po->DATA.data, po->DATA.len, "\r\nStrict-Transport-Security:", slen)) {
+		return -ENOTFOUND;
+	}
+
+	ptr = po->DATA.data;
+	end = ptr + po->DATA.len;
+
+	len = end - ptr;
+
+	ptr = (u_char*)memmem(ptr, len, "\r\nStrict-Transport-Security:", slen);
+	ptr += 2;
+
+	h_end = (u_char*)memmem(ptr, len, "\r\n", 2);
+	h_end += 2;
+
+	size_t before_header = ptr - po->DATA.data;
+	size_t header_length = h_end - ptr;
+	size_t new_len = 0;
+
+	u_char *new_html;
+	SAFE_CALLOC(new_html, len, sizeof(u_char));
+
+	BUG_IF(new_html == NULL);
+
+	memcpy(new_html, po->DATA.data, before_header);
+	new_len += before_header;
+
+	memcpy(new_html+new_len, h_end, (len - header_length) - before_header);
+	new_len += (len - header_length) - before_header;
+
+
+	memset(po->DATA.data, '\0', po->DATA.len);
+
+	memcpy(po->DATA.data, new_html, new_len);
+	po->DATA.len = new_len;
+
+	po->flags |= PO_MODIFIED;
+
+
+	return ESUCCESS;
+
+}
+
 /*******************************************/
 /* Sessions' stuff for ssl packets */
 
@@ -1083,7 +1280,7 @@ static size_t sslw_create_ident(void **i, struct packet_object *po)
    /* return the ident */
    *i = ident;
 
-   /* return the lenght of the ident */
+   /* return the length of the ident */
    return sizeof(struct sslw_ident);
 }
 
@@ -1134,7 +1331,6 @@ static void sslw_create_session(struct ec_session **s, struct packet_object *po)
    /* alloc of data elements */
    SAFE_CALLOC((*s)->data, 1, sizeof(struct ip_addr));
 }
-
 #endif /* HAVE_OPENSSL */
 
 /* EOF */
