@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_curses_view_connections.c,v 1.21 2004/07/12 19:57:27 alor Exp $
 */
 
 #include <ec.h>
@@ -29,9 +28,6 @@
 #include <ec_format.h>
 #include <ec_inject.h>
 
-#ifndef OS_WINDOWS
-   #include <sys/mman.h>
-#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -205,7 +201,7 @@ static void curses_connection_detail(void *conn)
    wdg_window_print(wdg_conn_detail, 1, 10, "Source port             :  %-5d  %s", ntohs(c->co->L4_addr1), service_search(c->co->L4_addr1, c->co->L4_proto));
    wdg_window_print(wdg_conn_detail, 1, 11, "Destination port        :  %-5d  %s", ntohs(c->co->L4_addr2), service_search(c->co->L4_addr2, c->co->L4_proto));
    
-   wdg_window_print(wdg_conn_detail, 1, 13, "Transferred bytes       :  %d", c->co->xferred);
+   wdg_window_print(wdg_conn_detail, 1, 13, "--> %d    <-- %d   total: %d ", c->co->tx, c->co->rx, c->co->xferred);
    
    if (c->co->DISSECTOR.user) {
       wdg_window_print(wdg_conn_detail, 1, 15, "Account                 :  %s / %s", c->co->DISSECTOR.user, c->co->DISSECTOR.pass);
@@ -235,7 +231,7 @@ static void curses_connection_data(void *conn)
    curr_conn = c->co;
    curr_conn->flags |= CONN_VIEWING;
    
-   /* default is splitted view */
+   /* default is split view */
    curses_connection_data_split();
 }
 
@@ -243,7 +239,7 @@ static void curses_connection_data_help(void)
 {
    char help[] = "HELP: shortcut list:\n\n"
                  "  ARROWS - switch between panels\n" 
-                 "    j    - switch from splitted to joined view\n"
+                 "    j    - switch from split to joined view\n"
                  "    y    - inject characters interactively\n"
                  "    Y    - inject characters from a file\n"
                  "    k    - kill the connection";
@@ -280,14 +276,14 @@ static void curses_connection_data_split(void)
    wdg_set_size(wdg_conndata, 1, 2, -1, SYSMSG_WIN_SIZE - 1);
    
    wdg_create_object(&wdg_c1, WDG_SCROLL, 0);
-   sprintf(title, "%s:%d", ip_addr_ntoa(&curr_conn->L3_addr1, tmp), ntohs(curr_conn->L4_addr1));
+   snprintf(title, MAX_ASCII_ADDR_LEN+6, "%s:%d", ip_addr_ntoa(&curr_conn->L3_addr1, tmp), ntohs(curr_conn->L4_addr1));
    wdg_set_title(wdg_c1, title, WDG_ALIGN_LEFT);
    wdg_set_color(wdg_c1, WDG_COLOR_TITLE, EC_COLOR_TITLE);
    wdg_set_color(wdg_c1, WDG_COLOR_FOCUS, EC_COLOR_FOCUS);
    wdg_set_size(wdg_c1, 2, 3, current_screen.cols / 2, SYSMSG_WIN_SIZE - 2);
    
    wdg_create_object(&wdg_c2, WDG_SCROLL, 0);
-   sprintf(title, "%s:%d", ip_addr_ntoa(&curr_conn->L3_addr2, tmp), ntohs(curr_conn->L4_addr2));
+   snprintf(title, MAX_ASCII_ADDR_LEN+6, "%s:%d", ip_addr_ntoa(&curr_conn->L3_addr2, tmp), ntohs(curr_conn->L4_addr2));
    wdg_set_title(wdg_c2, title, WDG_ALIGN_LEFT);
    wdg_set_color(wdg_c2, WDG_COLOR_TITLE, EC_COLOR_TITLE);
    wdg_set_color(wdg_c2, WDG_COLOR_FOCUS, EC_COLOR_FOCUS);
@@ -418,7 +414,7 @@ static void curses_connection_data_join(void)
    wdg_set_size(wdg_conndata, 1, 2, -1, SYSMSG_WIN_SIZE - 1);
    
    wdg_create_object(&wdg_join, WDG_SCROLL, 0);
-   sprintf(title, "%s:%d - %s:%d", ip_addr_ntoa(&curr_conn->L3_addr1, src), ntohs(curr_conn->L4_addr1),
+   snprintf(title, 64, "%s:%d - %s:%d", ip_addr_ntoa(&curr_conn->L3_addr1, src), ntohs(curr_conn->L4_addr1),
                                  ip_addr_ntoa(&curr_conn->L3_addr2, dst), ntohs(curr_conn->L4_addr2));
    wdg_set_title(wdg_join, title, WDG_ALIGN_LEFT);
    wdg_set_color(wdg_join, WDG_COLOR_TITLE, EC_COLOR_TITLE);
@@ -630,16 +626,16 @@ static void inject_file(char *path, char *file)
    char *filename;
    int fd;
    void *buf;
-   size_t size;
+   size_t size, ret;
    
    DEBUG_MSG("inject_file %s/%s", path, file);
    
    SAFE_CALLOC(filename, strlen(path)+strlen(file)+2, sizeof(char));
 
-   sprintf(filename, "%s/%s", path, file);
+   snprintf(filename, strlen(path)+strlen(file)+2, "%s/%s", path, file);
 
    /* open the file */
-   if ((fd = open(filename, O_RDONLY)) == -1) {
+   if ((fd = open(filename, O_RDONLY | O_BINARY)) == -1) {
       ui_error("Can't load the file");
       return;
    }
@@ -649,10 +645,18 @@ static void inject_file(char *path, char *file)
    /* calculate the size of the file */
    size = lseek(fd, 0, SEEK_END);
    
-   /* map it to the memory */
-   buf = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0);
-   if (buf == MAP_FAILED) {
-      ui_error("Can't mmap the file");
+   /* load the file in memory */
+   SAFE_CALLOC(buf, size, sizeof(char));
+ 
+   /* rewind the pointer */
+   lseek(fd, 0, SEEK_SET);
+   
+   ret = read(fd, buf, size);
+   
+   close(fd);
+
+   if (ret != size) {
+      ui_error("Cannot read the file into memory");
       return;
    }
 
@@ -663,8 +667,7 @@ static void inject_file(char *path, char *file)
       user_inject(buf, size, curr_conn, 2);
    }
 
-   close(fd);
-   munmap(buf, size);
+   SAFE_FREE(buf);
    
 }
 
