@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_gtk_view.c,v 1.10 2004/05/21 14:25:22 alor Exp $
 */
 
 #include <ec.h>
@@ -412,22 +411,24 @@ void gtkui_vis_method(void)
       }
 
       /* set vmethod string */
+      int i=0;
       switch(active) {
-         case 6: strcpy(vmethod, "hex"); break;
-         case 5: strcpy(vmethod, "ascii"); break; 
-         case 4: strcpy(vmethod, "text"); break;
-         case 3: strcpy(vmethod, "ebcdic"); break;
-         case 2: strcpy(vmethod, "html"); break;
+         case 6: strncpy(vmethod, "hex", 3); break;
+         case 5: strncpy(vmethod, "ascii", 5); break; 
+         case 4: strncpy(vmethod, "text", 4); break;
+         case 3: strncpy(vmethod, "ebcdic", 6); break;
+         case 2: strncpy(vmethod, "html", 4); break;
          case 1: /* utf8 */
             /* copy first word from encoding choice */
-            sscanf(gtk_entry_get_text(GTK_ENTRY (GTK_COMBO (lang_combo)->entry)),
+            i=sscanf(gtk_entry_get_text(GTK_ENTRY (GTK_COMBO (lang_combo)->entry)),
                    "%[^ ]", encoding);
+            BUG_IF(i!=1);
             if(strlen(encoding) > 0) {
-               strcpy(vmethod, "utf8");
+               strncpy(vmethod, "utf8", 4);
                set_utf8_encoding(encoding);
                break;
             }
-         default: strcpy(vmethod, "ascii");
+         default: strncpy(vmethod, "ascii", 5);
       }
 
       set_format(vmethod);
