@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: ec_ip.c,v 1.43 2004/09/28 09:56:13 alor Exp $
 */
 
 #include <ec.h>
@@ -115,8 +114,10 @@ FUNC_DECODER(decode_ip)
    ip_addr_init(&PACKET->L3.dst, AF_INET, (char *)&ip->daddr);
    
    /* this is needed at upper layer to calculate the tcp payload size */
+   /* check bogus size */
    t_len = (u_int32) ntohs(ip->tot_len);
-   if (t_len < (u_int32)DECODED_LEN)
+   if (t_len < (u_int32)DECODED_LEN || 
+       (DECODE_DATA + t_len) > (PACKET->packet + PACKET->len) )
       return NULL;
    PACKET->L3.payload_len = t_len - DECODED_LEN;
 
@@ -182,7 +183,7 @@ FUNC_DECODER(decode_ip)
    }
 
    /* calculate if the dest is local or not */
-   switch (ip_addr_is_local(&PACKET->L3.src)) {
+   switch (ip_addr_is_local(&PACKET->L3.src, NULL)) {
       case ESUCCESS:
          PACKET->PASSIVE.flags &= ~FP_HOST_NONLOCAL;
          PACKET->PASSIVE.flags |= FP_HOST_LOCAL;
@@ -200,7 +201,7 @@ FUNC_DECODER(decode_ip)
    hook_point(HOOK_PACKET_IP, po);
 
    /* don't save the sessions in unoffensive mode */
-   if (!GBL_OPTIONS->unoffensive && !GBL_OPTIONS->read) {
+   if (GBL_FILTERS && !GBL_OPTIONS->unoffensive && !GBL_OPTIONS->read) {
    
       /* Find or create the correct session */
       ip_create_ident(&ident, PACKET);
@@ -223,7 +224,7 @@ FUNC_DECODER(decode_ip)
    EXECUTE_DECODER(next_decoder);
    
    /* don't save the sessions in unoffensive mode */
-   if (!GBL_OPTIONS->unoffensive && !GBL_OPTIONS->read && (PACKET->flags & PO_FORWARDABLE)) {
+   if (GBL_FILTERS && !GBL_OPTIONS->unoffensive && !GBL_OPTIONS->read && (PACKET->flags & PO_FORWARDABLE)) {
       /* 
        * Modification checks and adjustments.
        * - ip->id according to number of injected/dropped packets
@@ -280,7 +281,7 @@ FUNC_INJECTOR(inject_ip)
    iph->tos      = 0;
    iph->csum     = CSUM_INIT;
    iph->frag_off = 0;            
-   iph->ttl      = 125;   
+   iph->ttl      = 64;   
    iph->protocol = PACKET->L4.proto; 
    iph->saddr    = ip_addr_to_int32(PACKET->L3.src.addr);   
    iph->daddr    = ip_addr_to_int32(PACKET->L3.dst.addr);   
@@ -330,7 +331,9 @@ FUNC_INJECTOR(inject_ip)
       PACKET->fwd_packet = PACKET->packet;
       PACKET->fwd_len = PACKET->L3.len;
    }
-   
+ 
+   /* Injectors executed, no need to keep the session */ 
+   session_del(s->ident, IP_IDENT_LEN); 
    return ESUCCESS;
 }
 
@@ -381,7 +384,7 @@ size_t ip_create_ident(void **i, struct packet_object *po)
    /* return the ident */
    *i = ident;
 
-   /* return the lenght of the ident */
+   /* return the length of the ident */
    return sizeof(struct ip_ident);
 }
 
